from huhk.testcase.apache.data import data_list_str4, data_type, data_list_str5

from huhk.unit_apache_beam import ApacheFun


class TestRegex:
    # 正则表达式匹配
    def test_regex_001(self):
        ApacheFun(data_list_str4, data_type=data_type).regex_matches(r'(?P<icon>[^\s,]+), *(\w+), *(\w+)').print()

    # 正则表达式与所有组匹配
    def test_regex_002(self):
        ApacheFun(data_list_str4, data_type=data_type).regex_all_match(r'(?P<icon>[^\s,]+), *(\w+), *(\w+)').print()

    # 正则表达式匹配成键值对
    def test_regex_003(self):
        ApacheFun(data_list_str4, data_type=data_type).regex_matches_kv(r'(?P<icon>[^\s,]+), *(\w+), *(\w+)', 'icon').print()

    # 正则表达式查找
    def test_regex_004(self):
        ApacheFun(data_list_str4, data_type=data_type).regex_find(r'(?P<icon>[^\s,]+), *(\w+), *(\w+)').print()

    # 正则表达式查找所有
    def test_regex_005(self):
        ApacheFun(data_list_str5, data_type=data_type).regex_find_all(r'(?P<icon>[^\s,]+), *(\w+), *(\w+)').print()

    # 正则表达式查找为键值对
    def test_regex_006(self):
        ApacheFun(data_list_str5).regex_find_kv(r'(?P<icon>[^\s,]+), *(\w+), *(\w+)', 'icon').print()

    # 正则表达式替换
    def test_regex_007(self):
        ApacheFun(data_list_str5, data_type=data_type).regex_replace_all(r'\s*,\s*', ':').print()

    # 先替换正则表达式
    def test_regex_008(self):
        ApacheFun(data_list_str5, data_type=data_type).regex_replace_first(r'\s*,\s*', ': ').print()

    # 正则表达式拆分
    def test_regex_009(self):
        ApacheFun(data_list_str5, data_type=data_type).regex_split(r'\s*,\s*').print()



import apache_beam as beam
from apache_beam.transforms import window


class SplitWords(beam.DoFn):
    def __init__(self, delimiter=','):
        self.delimiter = delimiter

    def process(self, text):
        for word in text.split(self.delimiter):
            yield word


class AnalyzeElement(beam.DoFn):
  def process(
      self,
      elem,
      timestamp=beam.DoFn.TimestampParam,
      window=beam.DoFn.WindowParam):
    yield '\n'.join([
        '# timestamp',
        'type(timestamp) -> ' + repr(type(timestamp)),
        'timestamp.micros -> ' + repr(timestamp.micros),
        'timestamp.to_rfc3339() -> ' + repr(timestamp.to_rfc3339()),
        'timestamp.to_utc_datetime() -> ' + repr(timestamp.to_utc_datetime()),
        '',
        '# window',
        'type(window) -> ' + repr(type(window)),
        'window.start -> {} ({})'.format(
            window.start, window.start.to_utc_datetime()),
        'window.end -> {} ({})'.format(
            window.end, window.end.to_utc_datetime()),
        'window.max_timestamp() -> {} ({})'.format(
            window.max_timestamp(), window.max_timestamp().to_utc_datetime()),
    ])


class DoFnMethods(beam.DoFn):
  def __init__(self):
    print('__init__')
    self.window = window.GlobalWindow()

  def setup(self):
    print('setup')

  def start_bundle(self):
    print('start_bundle')

  def process(self, element, window=beam.DoFn.WindowParam):
    self.window = window
    yield '* process: ' + element

  def finish_bundle(self):
    yield beam.utils.windowed_value.WindowedValue(
        value='* finish_bundle: 🌱🌳🌍',
        timestamp=0,
        windows=[self.window],
    )

  def teardown(self):
    print('teardown')


from huhk import projects_path, admin_host
from huhk.unit_dict import Dict
from huhk.unit_data import size_names, page_names, before_names, end_names, page_and_size


class ProjectInIt:
    def __init__(self, name=None, app_key=None, yapi_url=None, yapi_token=None, yapi_json_file=None, swagger_url=None):
        """api_type: 0时，value是swagger的api，json的url
                     1时，value值为yapi项目token,
                     2时，value是yapi下载的json文件名，文件放在file目录下,
                     3时，value是yapi的yapi-swagger.json
           name:项目名称，空时默认当前py文件所在文件名上级目录
        """
        self.path = Dict()
        self.path.dir = projects_path
        self.app_key = app_key
        self.name = name
        self.name2 = None
        self.yapi_url = yapi_url
        self.yapi_token = yapi_token
        self.yapi_json_file = yapi_json_file
        self.swagger_url = swagger_url
        self.url = admin_host
        self.yapi_file_str = None
        self.size_names = size_names
        self.page_names = page_names
        self.before = before_names
        self.end = end_names
        self.page_and_size = page_and_size
        self.this_file_list = Dict({"apis": []})
        self.this_fun_list = Dict()
        self.api_list = Dict()
        self.base_path = ""



size_names = ("pagesize", "size", "limit")
page_names = ("pageNum", "current", "currentpage", "page")
before_names = ("before", "start")
end_names = ("end", )
page_and_size = size_names + page_names
before_and_end = before_names + end_names
before_and_end_re_str = "|".join(["^" + i for i in before_and_end] + [i + "$" for i in before_and_end])



import random

from huhk.testcase.apache.data import data_list_str2, data_type, data_list_dict

from huhk.unit_apache_beam import ApacheFun
from testcase.apache.beam_class import SplitWords, AnalyzeElement, DoFnMethods


class TestParDo:
    # 有简单 DoFn 的 ParDo
    def test_par_do_001(self):
        ApacheFun(data_list_str2, data_type=data_type).par_do(SplitWords(',')).print()

    # 带有时间戳和窗口信息的 ParDo
    def test_par_do_002(self):
        af = ApacheFun([':)'])
        af.map(lambda elem: af.window.TimestampedValue(elem, 1584675660))
        af.window_info(af.window.FixedWindows(30))
        af.par_do(AnalyzeElement()).print()

    # ParDo 与 DoFn 方法
    def test_par_do_003(self):
        ApacheFun(['🍓', '🥕', '🍆', '🍅', '🥔'], data_type=data_type).par_do(DoFnMethods()).print()

    # 使用函数进行分区
    def test_par_do_004(self):
        durations = ['annual', 'biennial', 'perennial']

        def by_duration(plant, num_partitions):
            return durations.index(plant['duration']) if plant['duration'] in durations else len(durations)

        out = ApacheFun(data_list_dict, data_type=data_type).partition(by_duration, 4)
        # out.print("'{}: {}'.format(x['duration'], x)")
        out.value[0].print("'annual: {}'.format(x)")
        out.value[1].print("'biennial: {}'.format(x)")
        out.value[2].print("'perennial: {}'.format(x)")
        out.create_or_update_project()

    # 使用 lambda 函数进行分区
    def test_par_do_005(self):
        durations = ['annual', 'biennial', 'perennial']
        out = ApacheFun(data_list_dict, data_type=data_type).partition(
            lambda plant, num_partitions:
            durations.index(plant['duration']) if plant['duration'] in durations else len(durations), 4)
        out.value[0].print("'annual: {}'.format(x)")
        out.value[1].print("'biennial: {}'.format(x)")
        out.value[2].print("'perennial: {}'.format(x)")
        out.create_or_update_project()

    # 具有多个参数的分区
    def test_par_do_006(self):

        def split_dataset(plant, num_partitions, ratio):
            assert num_partitions == len(ratio)
            bucket = random.randint(0, sum(ratio)-1)
            total = 0
            for i, part in enumerate(ratio):
                total += part
                if bucket < total:
                    return i
            return len(ratio) - 1

        out = ApacheFun(data_list_dict*3).partition(split_dataset, 2, ratio=[10, 20])
        out.value[0].print("'1: {}'.format(x)")
        out.value[1].print("'2: {}'.format(x)")
        out.create_or_update_project()



import os.path
import re

from huhk.case_project.project_base import ProjectBase
from huhk.unit_dict import Dict
from huhk.unit_fun import FunBase
from huhk import projects_path


class GetApi(ProjectBase):
    def __init__(self, name=None, app_key=None, yapi_url=None, yapi_token=None, yapi_json_file=None, swagger_url=None):
        super().__init__(name, app_key, yapi_url, yapi_token, yapi_json_file, swagger_url)

    def create_or_update_project(self, _update=False):
        """
        创建项目
        """
        # 创建项目目录
        self.set_file_path()
        # 获取已维护api方法接口列表
        self.get_this_fun_list()
        # 获取接口文档api接口列表
        self.get_api_list()
        # 添加api封装方法
        self.write_fun(_update=_update)

    def set_file_path(self):
        """创建项目框架"""
        FunBase.mkdir_file(self.path.values(), is_py=True)
        if not os.path.exists(os.path.join(self.path.service_dir, "__init__.py")) or \
                not FunBase.read_file(os.path.join(self.path.service_dir, "__init__.py")):
            FunBase.write_file(os.path.join(self.path.service_dir, "__init__.py"), value=self.get_init_value())
        if not os.path.exists(os.path.join(self.path.service_dir, f"{self.name}_fun.py")):
            FunBase.write_file(os.path.join(self.path.service_dir, f"{self.name}_fun.py"), value=self.get_fun_value())
        if not os.path.exists(os.path.join(self.path.service_dir, f"setting.json")):
            FunBase.write_file(os.path.join(self.path.service_dir, f"setting.json"), value="{}")
        if not os.path.exists(os.path.join(self.path.testcase_dir, f"conftest.py")):
            FunBase.write_file(os.path.join(self.path.testcase_dir, f"conftest.py"), value=self.get_conftest_value())

    def get_api_list(self):
        """根据api文档不同方式生成api文件"""
        if self.swagger_url:
            self.get_list_menu_swagger()
        elif self.yapi_url and self.yapi_token:
            self.get_list_menu()
        elif self.yapi_file_str or self.yapi_json_file:
            self.get_list_json()

    def write_fun(self, _update=False):
        if not self.this_fun_list.api and not self.api_list:
            self.api_list += [{'method': 'GET', 'title': '示例-get', 'path': '/demo/get', 'up_time': 1675665418},
                              {'method': 'POST', 'title': '示例-post', 'path': '/demo/post', 'up_time': 1675665418}]
        for row in self.api_list:
            self.write_api(row)
        for fun_name in self.this_fun_list.api.keys():
            self.write_sql(fun_name, _update=_update)
            self.write_assert(fun_name, _update=_update)
            self.write_api_fun(fun_name, _update=_update)
            self.write_testcase(fun_name, _update=_update)

    def sub_hz(self, _id, _str):
        if re.findall(r'[^\da-zA-Z_\ (=,*):]', re.findall(r"def .*?\):", _str)[0]):
            api = self.get_api(_id)
            tmp2 = api.get('data', {}).get('req_params', [])
            for tmp3 in tmp2:
                name = tmp3.get('name', "")
                desc = tmp3.get('desc', "")
                if re.findall(r'[^\da-zA-Z_\ (=,*):]', name) and not re.findall(r'\W', desc):
                    _str = _str.replace(name, desc)
        if re.findall(r'[( ]async[,=)]', _str):
            for tmp in re.findall(r'[( ]async[,=)]', _str):
                tmp1 = str(tmp).replace('async', 'async1')
                _str = _str.replace(tmp, tmp1)
        return _str

    @staticmethod
    def get_testcase_path():
        testcase_path = GetApi.get_service_value("testcase_path")
        if not testcase_path or not os.path.exists(testcase_path):
            testcase_path = os.path.join(ProjectBase._get_project_dir(), "testcase")
            GetApi.set_service_value("testcase_path", testcase_path)
        return testcase_path

    @staticmethod
    def get_report_path():
        report_dir = os.path.join(ProjectBase._get_project_dir(), "report", FunBase.data_str())
        FunBase.mkdir_file(report_dir, is_py=False)
        report_path = os.path.join(report_dir, "%03d_%s" % (len(os.listdir(report_dir)) + 1, FunBase.time_str("%H%M%S")))
        FunBase.mkdir_file(report_path, is_py=False)
        return report_path

    @staticmethod
    def get_report_json(report_path=None):
        report_dir = report_path or GetApi.get_report_path()
        path = os.path.join(report_dir, "json")
        path2 = os.path.join(report_dir, "report_html")
        FunBase.mkdir_file([path, path2], is_py=False)
        return path, path2

    @staticmethod
    def get_key_name_list(path=None):
        out = Dict()
        path = path or projects_path
        for dirpath, dirnames, filenames in os.walk(path):
            if "apis" in dirnames and "asserts" in dirnames and "funs" in dirnames and "sqls" in dirnames \
                    and "__init__.py" in filenames:
                name = os.path.basename(dirpath)
                init_str = FunBase.read_file(os.path.join(dirpath, "__init__.py"))
                app_key = re.findall(r'\nAPP_KEY *= *[\'\"](.+)[\'\"]', init_str)
                app_key = app_key[0] if app_key else None
                if app_key:
                    out[name] = app_key
                    out[app_key] = name
                else:
                    out[name] = None
        if out:
            return out
        else:
            if os.path.exists(os.path.dirname(path)) and len(path) > 5:
                return GetApi.get_key_name_list(os.path.dirname(path))
        return out

    @staticmethod
    def get_running_path(path=None, testcase_dir=None):
        testcase_dir = testcase_dir or GetApi.get_testcase_path()
        if testcase_dir and path:
            path_list = [i for i in re.split(r"/|\\\\|\\", path) if i]
            for dirpath, dirnames, filenames in os.walk(testcase_dir):
                if path_list[0] in dirnames or path_list[0] in filenames:
                    if os.path.exists(os.path.join(dirpath, *path_list)):
                        return os.path.join(dirpath, *path_list)
                    elif len(path_list) > 1:
                        return GetApi.get_running_path(os.path.join(*path_list[1:]), os.path.join(dirpath, path_list[0]))
        print(f"用例目录{path}不存在")
        return None


if __name__ == '__main__':
    ga = GetApi()
    print(ga.get_running_path("yhzx/web/drop"))


import os
import re
import sys

import click

from huhk import allure_bat
from huhk.case_project.version import version as _version
from huhk.init_project import GetApi


def get_version():
    k = str(GetApi.get_service_value("this_key"))
    v = str(GetApi.get_service_value("this_name"))
    out_str = f"版本：{_version}\n--key：{k}\n--name：{v}"
    return out_str


def set_key_name(key, name):
    if key and name:
        GetApi.set_service_value("this_key", key)
        GetApi.set_service_value("this_name", name)
    elif key or name:
        key_list = GetApi.get_key_name_list()
        if key:
            GetApi.set_service_value("this_key", key)
            if key_list.get(key):
                GetApi.set_service_value("this_name", key_list.get(key))
        else:
            if key_list.get(name):
                GetApi.set_service_value("this_key", key_list.get(name))
            GetApi.set_service_value("this_name", name)
    return True


def install_project(app_key, name=None):
    ga = GetApi(name=name, app_key=app_key)
    ga.create_or_update_project()
    set_key_name(app_key, name)
    return "项目创建成功"


def update_project(app_key=None, name=None):
    set_key_name(app_key, name)
    app_key = GetApi.get_service_value("this_key")
    name = GetApi.get_service_value("this_name")
    if not app_key and not name:
        return "项目未指定，请指定参数-k/-n"
    else:
        ga = GetApi(name=name, app_key=app_key)
        ga.create_or_update_project()
        return "项目更新成功"


def fun_project(app_key=None, name=None, fun_url=None, method=None):
    set_key_name(app_key, name)
    app_key = GetApi.get_service_value("this_key")
    name = GetApi.get_service_value("this_name")
    if not app_key and not name:
        return "项目未指定，请指定参数-k/-n"
    else:
        ga = GetApi(name=name, app_key=app_key)
        methods = {1: "GET", 2: "POST", 3: "PUT", 4: "DELETE",
                   5: "HEAD", 6: "OPTIONS", 7: "PATCH", 8: "CONNECT"}
        while not method:
            method = input("输入方法类型（回车默认：get）：")
            if method == "":
                method = "GET"
            elif method.isdigit() and int(method) < 9:
                method = methods[int(method)]
            elif method.upper() in methods.values():
                method = method.upper()
            else:
                print(f"""输入类型错误，请重新输入：{methods}""")
                method = None
        if method == "POST":
            headers_list = {0: {"Content-Type": "application/json"},
                            1: {"Content-Type": "application/x-www-form-urlenc"},
                            2: {"Content-Type": "multipart/form-data"},
                            3: {"Content-Type": "text/xml"}}
            while not headers:
                headers = input("输入post请求类型（回车默认：application/json）：")
                if method == "":
                    headers = headers_list[0]
                elif method.isdigit() and int(method) < 4:
                    headers = headers_list[int(method)]
                else:
                    try:
                        headers = eval(headers)
                    except:
                        print(f"""输入类型错误，请重新输入\n""")
                        headers = None

        return "方法添加/更新成功"


def running_testcase(running, case_path=None, report_path=None):
    running_path_list = []
    for cases in running:
        for case in re.split(r'[，,;；\n\t ]', cases):
            running_path = GetApi.get_running_path(case, case_path)
            if running_path:
                running_path_list.append(running_path)
    if running_path_list:
        report_path, report_html = GetApi.get_report_json(report_path)
        running_setting = ["-s", "-v", "--alluredir", report_path, "-p", "no:warnings", "--cache-clear",
                              "--reruns=1", "--reruns-delay=1", "--instafail", "--tb=line"]
        cmd_str = "%s -m pytest %s" % (sys.executable, " ".join(running_path_list + running_setting))
        print(cmd_str)
        os.system(cmd_str)

        if sys.platform == "win32":
            cmd_str = f"call {allure_bat} generate {report_path} -c -o {report_html}"
        else:
            cmd_str = f"allure generate {report_path} -c -o {report_html}"
        print(cmd_str)
        os.system(cmd_str)
        if not case_path and not report_path and sys.platform == "win32":
            os.system(f"call {allure_bat} open {report_html}")
        return "执行用例完成"
    else:
        return "无可执行用例文件"


if __name__ == '__main__':
    # install_project('e7f7ea25-aa2f-40b9-bd4c-1277b3173ae8')
    running_testcase(["yhzx/web/drop"], case_path=r"D:\projects\python_test\ardar_test\autotest\codes\1\testcase",
                     report_path=r"D:\projects\python_test\huhk-common\report\test6")


import click


from huhk.case_project.main_fun import get_version, set_key_name, install_project, update_project, fun_project, \
    running_testcase


@click.command()
@click.option('-v', '--version', help='版本, 当前--key/--name', is_eager=True, is_flag=True)
@click.option('-k', '--key', help='项目key, 传*表示所有项目，不传默认去上一次key')
@click.option('-n', '--name', help='项目名称, 传*表示所有项目，不传默认去上一次name')
@click.option('-i', '--install', help='根据key创建项目, 项目存在则更新')
@click.option('-u', '--update', help='根据--key更新项目，--key不存在时默认取之前的值', is_eager=True, is_flag=True)
@click.option('-f', '--fun', help='新增api方法，参数url，--key不存在时默认取之前的值')
@click.option('-r', '--running', help='执行测试用例, 项目或目录', multiple=True)
@click.option('-cp', '--casepath', help='生成的测试报告文件路径')
@click.option('-rp', '--reportpath', help='生成的测试报告文件路径')
def main(version, key, name, install, update, fun, running, casepath, reportpath):
    print(version, key, name, install, update, fun, running, casepath, reportpath)
    if version:
        click.echo(get_version())
    else:
        set_key_name(key, name)
        if install:
            click.echo(install_project(install, name))
        elif update:
            click.echo(update_project(key, name))
        elif fun:
            click.echo(fun_project(app_key=key, name=name, fun_url=fun))
        elif running:
            click.echo(running_testcase(running, casepath, reportpath))


if __name__ == '__main__':
    main()



import os
import sys
admin_host = "http://47.96.124.102/api"
# admin_host = "http://127.0.0.1:8000/api"

projects_path = os.getcwd()
file_path = os.path.abspath(__file__)
dir_path = os.path.dirname(file_path)
allure_bat = os.path.join(dir_path, "allure-2.23.1", "bin", "allure.bat")

projects_path_list = projects_path.split(os.path.sep)
if "site-packages" in projects_path_list and "huhk" in projects_path_list:
    projects_path = os.path.join(*projects_path_list[:projects_path_list.index("site-packages") - 2])
elif "service" in projects_path_list:
    projects_path = os.path.join(*projects_path_list[:projects_path_list.index("service")])
elif "testcase" in projects_path_list:
    projects_path = os.path.join(*projects_path_list[:projects_path_list.index("testcase")])





import json
import os.path
import re
import time

import requests

from huhk.case_project.project_string import ProjectString
from huhk.unit_dict import Dict
from huhk.unit_fun import FunBase
from huhk import projects_path
from huhk.unit_logger import logger


class ProjectBase(ProjectString):
    def __init__(self, name=None, app_key=None, yapi_url=None, yapi_token=None, yapi_json_file=None, swagger_url=None):
        super().__init__(name, app_key, yapi_url, yapi_token, yapi_json_file, swagger_url)
        self.get_project()

    def get_service_value2(self, key=None, default=None):
        """获取项目本地变量"""
        this_path = self.get_setting_path2()
        value = ProjectBase.get_json_value(this_path, key=key, default=default)
        return value if value else ProjectBase.get_service_value(key=key, default=default)

    @staticmethod
    def get_service_value(key=None, default=None):
        path = ProjectBase.get_setting_path()
        return ProjectBase.get_json_value(path, key=key, default=default)

    @staticmethod
    def get_json_value(path, key=None, default=None):
        if os.path.exists(path):
            with open(path, encoding="utf-8") as fp:
                data = json.load(fp)
                return data.get(key, default) if key else data
        return default

    @staticmethod
    def set_json_value(path, key=None, value=None):
        FunBase.mkdir_file(path, is_py=False)
        data = ProjectBase.get_json_value(path) or {}
        data[key] = value
        with open(path, 'w') as fp:
            json.dump(data, fp, indent=4)

    @staticmethod
    def get_setting_path():
        setting_path = os.path.join(ProjectBase._get_project_dir(), "service", "setting.json")
        if not os.path.exists(setting_path):
            FunBase.mkdir_file(setting_path, is_py=False)
            FunBase.write_file(setting_path, "{}")
        return setting_path

    def get_setting_path2(self):
        setting_path = os.path.join(self.path.service_dir, self.name + "_setting.json")
        if not os.path.exists(setting_path):
            FunBase.mkdir_file(setting_path, is_py=False)
            FunBase.write_file(setting_path, "{}")
        return setting_path

    @staticmethod
    def _get_project_dir(path=None):
        path = path or projects_path
        for dirpath, dirnames, filenames in os.walk(path):
            if os.path.join("autotest", "codes") not in dirpath:
                if "apis" in dirnames and "asserts" in dirnames and "funs" in dirnames and "sqls" in dirnames \
                        and "__init__.py" in filenames:
                    return os.path.dirname(os.path.dirname(dirpath))
        if len(path) > 5:
            return ProjectBase._get_project_dir(os.path.dirname(path))
        return projects_path

    @staticmethod
    def set_service_value(key, value):
        ProjectBase.set_json_value(ProjectBase.get_setting_path(), key=key, value=value)

    def set_service_value2(self, key, value, _type=0):
        """设置项目本地变量"""
        ProjectBase.set_json_value(self.get_setting_path2(), key=key, value=value)

    def get_api_attribute(self, file_str, api_file):
        try:
            fun_list = re.findall(r"\ndef +([^_].+)?\((.*?)\):\s*"
                                  r"([\'\"]{3}"
                                  r"\s*(.*?)?\s*up_time=(\d+)?[\w\W]*?"
                                  r"(    params:[\w\W]*?)?(====[\w\W]*?)?"
                                  r"[\'\"]{3})?"
                                  r"[\w\W]*?_method *= *[\"\'](\S+?)[\"\']"
                                  r"[\w\W]*?_url *= *[\"\'](\S+?)[\"\']"
                                  r"[\w\W]*?(\n@allure.step\(|\ndef|$)", file_str)
            for fun in fun_list:
                self.this_fun_list.api[fun[0]].path = api_file
                self.this_fun_list.api[fun[0]].input = [i.split('=')[0].strip() for i in fun[1].split(',') if
                                                        len(i.split('=')) > 1 and i.strip()[0] != '_' and i.split('=')[
                                                            0].strip() != "headers"]
                self.this_fun_list.api[fun[0]].title = fun[3]
                self.this_fun_list.api[fun[0]].up_time = fun[4]
                self.this_fun_list.api[fun[0]].params = fun[5]
                self.this_fun_list.api[fun[0]].method = fun[7]
                self.this_fun_list.api[fun[0]].url = fun[8]
        except Exception as e:
            logger.error(str(e))

    def get_this_fun_list(self):
        try:
            last_time = self.get_service_value2("last_time")
            # self.set_service_value("last_time", time.time())
            service_dir = os.path.join(self.path.dir, 'service', self.name)
            for dirpath, dirnames, filenames in os.walk(service_dir):
                for filename in filenames:
                    key = filename.split("_")[0]
                    if key in self.this_file_list.keys() and filename[-3:] == ".py":
                        if not (key == "apis" and last_time and
                                os.stat(os.path.join(dirpath, filename)).st_mtime < last_time):
                            self.this_file_list[key].append(os.path.join(dirpath, filename))
            for api_file in self.this_file_list.get('apis'):
                file_str = FunBase.read_file(api_file)
                self.get_api_attribute(file_str, api_file)
        except Exception as e:
            print(str(e))

    def get_project(self, path=None):
        if self.app_key:
            project = requests.post(self.url + '/variable/variable/',
                                    json={"app_key": self.app_key, "environment": "sit"}).json()
            if project.get('project'):
                self.name = self.name or project.get('project')[0].get('code')
                if project.get('api_settings'):
                    self.api_type = project.get('api_settings')[0].get('api_type')
                    if self.api_type == 2:
                        self.yapi_file_str = requests.get(self.url + "/media/" +
                                                          project.get('api_settings')[0].get('file')).text
                    elif self.api_type in (0, 3):
                        self.swagger_url = self.swagger_url or project.get('api_settings')[0].get('url')
                    elif self.api_type == 1:
                        self.yapi_url = self.yapi_url or project.get('api_settings')[0].get('url').strip()
                        if self.yapi_url[-1] == "/":
                            self.yapi_url = self.yapi_url[:-1]
                        self.yapi_token = self.yapi_token or project.get('api_settings')[0].get('token')
            else:
                self.error = project.get('non_field_errors')[0]
                logger.error("该app_key: %s, 对应的项目不存在，项目创建：%s/admin/#/admin/autotest/project/", (
                    self.app_key, self.url[:-4]))
        self.name = self.name or "demo"
        self.name2 = self.name.title()
        projects_path = path or ProjectBase._get_project_dir()
        logger.info(f"项目路径：{projects_path}")
        self.path.service_dir = os.path.join(projects_path, "service", self.name)
        self.path.testcase_dir = os.path.join(projects_path, "testcase", self.name)
        self.path.api_dir = os.path.join(self.path.service_dir, "apis")
        self.path.fun_dir = os.path.join(self.path.service_dir, "funs")
        self.path.assert_dir = os.path.join(self.path.service_dir, "asserts")
        self.path.sql_dir = os.path.join(self.path.service_dir, "sqls")

    def get_list_menu_swagger(self):
        try:
            data = Dict(requests.get(self.swagger_url).json())
            for k, v in data.paths.items():
                api = Dict()
                api.path = data.basePath + k
                for k2, v2 in v.items():
                    api.method = k2
                    api.title = v2.get('summary', "")
                    api.up_time = int(time.time())
                    api.req_headers = [{'name': 'Content-Type', 'desc': '',
                                        'value': v2.consumes[
                                            0] if v2.consumes else "application/x-www-form-urlencoded"}]
                    api["req_params"] = []
                    api["req_query"] = []
                    api["req_body_other"] = []
                    api["res_body"] = []
                    for parameter in v2.get('parameters', []):
                        if parameter.get('in') == "body":
                            if parameter.get('name') not in ("params", "headers"):
                                if parameter.get("schema") and parameter.get("schema").get("$ref"):
                                    tmp = parameter.get("schema").get("$ref").split("/")
                                    if len(tmp) > 2:
                                        tmp2 = data.get(tmp[1], {}).get(tmp[2], {})
                                        for k3, v3 in tmp2.get('properties', {}).items():
                                            api["res_body"].append({'name': k3, 'desc': v3.get('description', "")})
                                else:
                                    api["res_body"].append({'name': parameter.get("name"),
                                                            'desc': parameter.get('description')})
                        elif parameter.get('in') in ("params", "path"):
                            api["req_params"].append({'name': parameter.get("name"),
                                                      'desc': parameter.get('description')})
                        elif parameter.get('in') == "query":
                            if parameter.get('name') not in ("params",):
                                api["req_query"].append({'name': parameter.get("name"),
                                                         'desc': parameter.get('description')})
                        else:
                            print("联系管理员维护类型：", parameter.get('in'))
                    self.api_list += [api]
        except Exception as e:
            logger.error("swagger获取接口失败")
            logger.error(str(e))
            self.api_list = []

    def get_list_menu(self):
        try:
            res = requests.get(self.yapi_url + "/api/project/get?token=" + self.yapi_token).text
            res_json = Dict(json.loads(res))
            base_path = res_json.data.basepath or ""
            data = {"token": self.yapi_token, "project_id": res_json.data.get("_id")}
            res = requests.get(self.yapi_url + "/api/interface/list_menu", data=data)
            res_json = Dict(json.loads(res.text))
            for menu in res_json.get("data"):
                self.api_list += menu.get('list')
            if base_path:
                self.base_path = base_path
                for api in self.api_list:
                    api.path = api.path
        except Exception as e:
            logger.error("yapi获取接口失败")
            logger.error(str(e))
            self.api_list = []

    def get_list_json(self):
        if self.yapi_json_file:
            file_path = os.path.join(self.path.dir, 'file', self.yapi_json_file)
            if os.path.exists(file_path):
                value = FunBase.read_file(file_path)
                value = json.loads(value)
                for v in value:
                    self.api_list += v.get('list')
            else:
                assert not "Yapi的json文件在file中不存在"
        elif self.app_key:
            value = json.loads(self.yapi_file_str)
            for v in value:
                self.api_list += v.get('list')

    def write_api(self, row, _update=True):
        fun_name = self.get_fun_name(row.get("path"))
        if fun_name in self.this_fun_list.api.keys():
            up_time = self.this_fun_list.api[fun_name].up_time
            if not up_time or int(up_time) < row.get('up_time'):
                api_file_str = FunBase.read_file(self.this_fun_list.api[fun_name].path)
                ord_str = re.findall(
                    r'\n((@allure.step\(.*\) *\n)?def %s\(.+\)[\w\W]*?)(\n@allure.step\(|\ndef|$)' % fun_name,
                    api_file_str)
                new_str = self.get_api_fun_str(fun_name, row)
                api_file_str = api_file_str.replace(ord_str[0][0], new_str)
                FunBase.write_file(self.this_fun_list.api[fun_name].path, api_file_str)
        else:
            self.this_fun_list.api[fun_name] = self.get_path(fun_name)
            if os.path.exists(self.this_fun_list.api[fun_name].path):
                api_file_str = FunBase.read_file(self.this_fun_list.api[fun_name].path)
            else:
                api_file_str = self.get_api_header_str(self.this_fun_list.api[fun_name].import_path)
            api_file_str += self.get_api_fun_str(fun_name, row)
            FunBase.mkdir_file(self.this_fun_list.api[fun_name].path, is_py=False)
            init_dir = os.path.dirname(self.this_fun_list.api[fun_name].path)
            for i in range(self.this_fun_list.api[fun_name].import_path.count('.') - 2):
                init_path = os.path.join(init_dir, "__init__.py")
                if not os.path.exists(init_path) or not FunBase.read_file(init_path):
                    FunBase.write_file(init_path, self.get_api_init_str(
                        ".".join(self.this_fun_list.api[fun_name].import_path.split(".")[:-2-i])))
                init_dir = os.path.dirname(init_dir)
            FunBase.write_file(self.this_fun_list.api[fun_name].path, api_file_str)
            self.get_api_attribute(api_file_str, self.get_path(fun_name).path)

    def write_sql(self, fun_name, _update=False):
        sql_path = self.get_path(fun_name, fun_type='sqls')
        if os.path.exists(sql_path.path):
            file_str = FunBase.read_file(sql_path.path)
            ord_str = re.findall(
                r'\n((    @allure.step\(.*\) *\n)?    def sql_%s\(.+\)[\w\W]*?)(\n    @allure.step\(|\n    def|$)' %
                fun_name, file_str)
            if ord_str:
                if _update:
                    new_str = self.get_sql_fun_str(fun_name)
                    file_str = file_str.replace(ord_str[0][0], new_str)
                    FunBase.write_file(sql_path.path, file_str)
            else:
                tmp_list = file_str.split("if __name__ == '__main__':")
                tmp_list[0] += self.get_sql_fun_str(fun_name)
                file_str = "if __name__ == '__main__':".join(tmp_list)
                FunBase.write_file(sql_path.path, file_str)
        else:
            file_str = self.get_sql_header_str(sql_path.class_name) + self.get_sql_fun_str(fun_name)
            FunBase.mkdir_file(sql_path.path)
            FunBase.write_file(sql_path.path, file_str)

    def write_assert(self, fun_name, _update=False):
        sql_path = self.get_path(fun_name, fun_type='sqls')
        assert_path = self.get_path(fun_name, fun_type='asserts')
        if os.path.exists(assert_path.path):
            file_str = FunBase.read_file(assert_path.path)
            ord_str = re.findall(
                r'\n((    @allure.step\(.*\) *\n)?    def assert_%s\(.+\)[\w\W]*?)(\n    @allure.step\(|\n    def|$)' %
                fun_name, file_str)
            if ord_str:
                if _update:
                    new_str = self.get_assert_fun_str(fun_name)
                    file_str = file_str.replace(ord_str[0][0], new_str)
                    FunBase.write_file(assert_path.path, file_str)
            else:
                tmp_list = file_str.split("if __name__ == '__main__':")
                tmp_list[0] += self.get_assert_fun_str(fun_name)
                file_str = "if __name__ == '__main__':".join(tmp_list)
                FunBase.write_file(assert_path.path, file_str)
        else:
            file_str = self.get_assert_header_str(assert_path.class_name, sql_path) + self.get_assert_fun_str(fun_name)
            FunBase.mkdir_file(assert_path.path)
            FunBase.write_file(assert_path.path, file_str)

    def set_api_fun_header(self, fun_name):
        fun_name = fun_name.strip(" _")

        def _fun_header():
            return f"from {fun_path.import_path} import {fun_path.class_name}\n\n\n" \
                   f"class {fun_path2.class_name}({fun_path.class_name}):\n    pass\n\n"

        fun_path = self.get_path(fun_name, fun_type="funs")
        if fun_name.count('_') > 0:
            # path = fun_path.path.rsplit('_', 1)[0] + '.py'
            fun_path2 = self.get_path(fun_name.rsplit('_', 1)[0], fun_type='funs')
            path = fun_path2.path
            if os.path.exists(path):
                old_str = FunBase.read_file(path)
                old_str_l = re.findall(r'([\w\W]*?\n)(\s*class +.*\()(.*?)(\):[\w\W]*)', old_str)
                if old_str_l:
                    old_str_l = list(old_str_l[0])
                    if fun_path.class_name not in [i.strip() for i in old_str_l[2].split(',')]:
                        old_str_l[0] += f"from {fun_path.import_path} import {fun_path.class_name}\n"
                        old_str_l[2] += f", {fun_path.class_name}"
                    new_str = "".join(old_str_l)
                else:
                    new_str = _fun_header()
            else:
                new_str = _fun_header()
            FunBase.write_file(path, new_str)
            if fun_name.count('_') > 1:
                self.set_api_fun_header(fun_name.rsplit('_', 1)[0])
        else:
            print("Warning")

    def write_api_fun(self, fun_name, _update=False):
        fun_path = self.get_path(fun_name, fun_type='funs')

        if os.path.exists(fun_path.path):
            file_str = FunBase.read_file(fun_path.path).replace("    pass\n\n", "").replace("    pass\n", "")
            ord_str = re.findall(
                r'\n((    @allure.step\(.*\) *\n)?    def %s\(.+\)[\w\W]*?)(\n    @allure.step\(|\n    def|$)' %
                fun_name, file_str)
            if ord_str:
                if _update:
                    new_str = self.get_api_fun_fun_str(fun_name)
                    file_str = file_str.replace(ord_str[0][0], new_str)
                    FunBase.write_file(fun_path.path, file_str)
            else:
                file_str = self.get_api_fun_header_str2(fun_name, file_str)
                tmp_list = file_str.split("if __name__ == '__main__':")
                tmp_list[0] += self.get_api_fun_fun_str(fun_name)
                file_str = "if __name__ == '__main__':".join(tmp_list)
                FunBase.write_file(fun_path.path, file_str)
        else:
            file_str = self.get_api_fun_header_str(fun_name) + self.get_api_fun_fun_str(fun_name)
            FunBase.mkdir_file(fun_path.path)
            FunBase.write_file(fun_path.path, file_str)
            self.set_api_fun_header(fun_name)

    def write_testcase(self, fun_name, _update=False):
        fun_path = self.get_path(fun_name, fun_type='test')
        testcase_str_list = self.get_api_testcase_str(fun_name)
        if os.path.exists(fun_path.path):
            file_str = FunBase.read_file(fun_path.path).replace("    pass\n", "")
            ord_str_list = re.findall(r'\n    def test_(%s(?:__.+?)?)(?:_(\d+))?\(.+\):\n' % fun_name, file_str)
            if ord_str_list:
                ord_list = {j[0]: (int(j[1]) if j[1] else i + 1) for i, j in enumerate(ord_str_list)}
                i = len(ord_str_list)
                for case_name, fun_str in testcase_str_list.items():
                    if case_name in ord_list.keys():
                        if _update:
                            old_str = re.findall(r'(( *#.*\n|    @.*\n)*    def test_%s(?:_\d+)?\(.+\):[\w\W]+?\n+)'
                                                 r'(?:( *#.*\n|    @.*\n)*    def |$)?' % case_name, file_str)
                            fun_str = fun_str.replace(f"def test_{case_name}(",
                                                      f"def test_{case_name}{'_%03d' % ord_list.get(case_name)}(")
                            file_str = file_str.replace(old_str[0][0], fun_str)
                    else:
                        i += 1
                        tmp_list = file_str.split("if __name__ == '__main__':")
                        tmp_list[0] += fun_str.replace(f"def test_{case_name}(",
                                                       f"def test_{case_name}{'_%03d' % i}(")
                        file_str = "if __name__ == '__main__':".join(tmp_list)
            else:
                i = 0
                for case_name, fun_str in testcase_str_list.items():
                    i += 1
                    tmp_list = file_str.split("if __name__ == '__main__':")
                    tmp_list[0] += fun_str.replace(f"def test_{case_name}(", f"def test_{case_name}{'_%03d' % i}(")
                    file_str = "if __name__ == '__main__':".join(tmp_list)
            FunBase.write_file(fun_path.path, file_str)
        else:
            file_str = self.get_api_testcase_header_str(fun_name)
            i = 1
            for case_name, fun_str in testcase_str_list.items():
                file_str += fun_str.replace(f"def test_{case_name}(", f"def test_{case_name}{'_%03d' % i}(")
                i += 1
            FunBase.mkdir_file(fun_path.path, is_py=False)
            FunBase.write_file(fun_path.path, file_str)


if __name__ == "__main__":
    a = ProjectBase(name="app_t")
    a.dir = r"/"
    o = a.get_setting_path()
    print(o)



import os
import sys

path = os.path.dirname(os.path.dirname(sys.argv[0]))

os.chdir(path)
os.system("rmdir /q /s dist")
os.system("rmdir /q /s build")
os.system("python huhk/case_project/setup_set.py sdist bdist_wheel")
os.system("python -m twine upload dist/*")
# os.system("pip install huhk -U -i https://pypi.org/project")



from huhk.testcase.apache.data import data_list_str, data_type, data_list_str2, data_list_list, data_list_tuple, \
    data_list_dict, data_list_dict2

from huhk.unit_apache_beam import ApacheFun


class TestFlatmap:
    def setup(self):
        self.af = ApacheFun(data_list_str, data_type=data_type)

    # 带有预定义函数的 FlatMap
    def test_flatmap_001(self):
        self.af.flat_map(str.split).print()

    # 带有函数的FlatMap
    def test_flatmap_002(self):
        self.af.set_data(data_list_str2)
        self.af.flat_map(lambda x: x.split(",")).print()

    # 带有生成器的FlatMap
    def test_flatmap_003(self):
        def generate_elements(elements):
            for element in elements:
                yield element
        self.af.set_data(data_list_list)
        self.af.flat_map(generate_elements).print()

    # 键值对的FlatMapTuple
    def test_flatmap_004(self):
        def format_plant(icon, plant):
            if icon:
                yield '{}{}'.format(icon, plant)
        self.af.set_data(data_list_tuple)
        self.af.flat_map_tuple(format_plant).print()

    # 带有多个参数的 FlatMap
    def test_flatmap_005(self):
        def split_words(text, delimiter=None):
            return text.split(delimiter)
        self.af.set_data(data_list_str2)
        self.af.flat_map(split_words, ",").print()

    # 将输入作为迭代器的 FlatMap
    def test_flatmap_006(self):
        def normalize_and_validate_durations(plant, valid_durations):
            plant['duration'] = plant['duration'].lower()
            if plant['duration'] in valid_durations:
                yield plant
        self.af.set_data(data_list_dict)
        self.af.flat_map(normalize_and_validate_durations, self.af.pvalue_as_iter(data_list)).print()

    # 将输入作为字典的 FlatMap
    def test_flatmap_007(self):
        def replace_duration_if_valid(plant, durations):
            if plant['duration'] in durations:
                plant['duration'] = durations[plant['duration']]
                yield plant
        self.af.set_data(data_list_dict2)
        self.af.flat_map(replace_duration_if_valid, self.af.pvalue_as_dict(data_list_tuple2)).print()